import logging

import requests
from django.conf import settings

from .base import BaseSmsClient

logger = logging.getLogger("notifier")


class CGSmsClient(BaseSmsClient):

    @classmethod
    def send(cls, number: str, text: str, **kwargs):
        sub_account = settings.NOTIFIER["SMS"]["GATEWAYS"]["CGS"]["SUB_ACCOUNT"]
        sub_account_pass = settings.NOTIFIER["SMS"]["GATEWAYS"]["CGS"]["SUB_ACCOUNT_PASSWORD"]
        params = {
            "sub_account": sub_account,
            "sub_account_pass": sub_account_pass,
            "action": "send_sms",
            "message": text,
            "recipients": number,
        }
        res = requests.get("http://cheapglobalsms.com/api_v1", params=params)
        return res



from django.core.mail import get_connection


class DjangoEmailClient:

    @classmethod
    def get_connection(cls, email_settings:dict):
        smtp_host = email_settings["HOST"]
        smtp_port = email_settings["PORT"]
        smtp_use_tls = email_settings.get("USE_TLS", False)
        smtp_use_ssl = email_settings.get("USE_SSL", False)
        smtp_username = email_settings["USER"]
        smtp_password = email_settings["PASSWORD"]

        return get_connection(
            host=smtp_host,
            port=smtp_port,
            username=smtp_username,
            password=smtp_password,
            use_tls=smtp_use_tls,
            use_ssl=smtp_use_ssl,
        )


import logging
import traceback
from threading import Thread

from django.template.loader import render_to_string

try:
    from asgiref.sync import async_to_sync
    from channels.layers import get_channel_layer
except:
    pass
import json

from django.template import Context, Template

from magic_notifier.utils import NotificationBuilder

logger = logging.getLogger("notif")


class Pusher:
    def __init__(
        self, subject, receivers: list, template: str, context: dict, **kwargs
    ):
        self.receivers: list = receivers
        self.template: str = template
        self.context: dict = context
        if 'subject' not in context and subject:
            self.context['subject'] = subject
        self.threaded: bool = kwargs.get("threaded", False)
        self.image = kwargs.get("image")

    def send(self):
        if self.threaded:
            t = Thread(target=self._send)
            t.setDaemon(True)
            t.start()
        else:
            return self._send()

    def _send(self):
        try:
            for user in self.receivers:
                logger.debug(f"sending push notification to {user}")
                ctx = self.context.copy()
                ctx["user"] = user
                push_content = render_to_string(f"notifier/{self.template}/push.json", ctx)

                event: dict = json.loads(push_content)
                event['type'] = 'notification'

                not_builder = (
                    NotificationBuilder(event["subject"])
                    .text(event['text'])
                    .type(event["type"], event["sub_type"])
                    .link(event["link"])
                    .mode(event["mode"])
                    .data(event["data"])
                    .actions(event["actions"])
                    .user(user)
                )

                if self.image:
                    not_builder.image(self.image)

                res = not_builder.save()
                event['id'] = res.id

                # return
                channel_layer = get_channel_layer()
                async_to_sync(channel_layer.group_send)(
                    f"user-{user.id}",
                    event
                )

                return res
        except:
            logger.error(traceback.format_exc())


if __name__ == "__main__":
    pass



# Generated by Django 2.2.17 on 2021-07-23 21:30

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import magic_notifier.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Notification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('type', models.CharField(max_length=30)),
                ('sub_type', models.CharField(blank=True, max_length=30, null=True)),
                ('link', models.CharField(max_length=255, verbose_name='The link associated')),
                ('image', models.ImageField(upload_to='notifications')),
                ('actions', magic_notifier.models.JSONField(default=dict)),
                ('data', magic_notifier.models.JSONField(default=dict)),
                ('read', models.DateTimeField(blank=True, null=True)),
                ('sent', models.DateTimeField(auto_now_add=True)),
                ('mode', models.CharField(choices=[('user', 'User'), ('admin', 'Admin')], default='user', max_length=10)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notifications', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



# Generated by Django 3.2.15 on 2022-09-07 21:12

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('magic_notifier', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='id',
            field=models.BigAutoField(primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='notification',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='magic_notifications', to=settings.AUTH_USER_MODEL),
        ),
        migrations.CreateModel(
            name='NotifyProfile',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False)),
                ('phone_number', models.CharField(blank=True, max_length=20, null=True)),
                ('current_channel', models.CharField(blank=True, max_length=255, null=True)),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import importlib
import logging
import traceback
from threading import Thread
from typing import Optional

from django.conf import settings
from django.template.loader import render_to_string

from magic_notifier.utils import get_settings, import_attribute

logger = logging.getLogger("notifier")


class ExternalSMS:

    def __init__(self, receivers: list, context: dict, template: Optional[str] = None,
                 final_message: Optional[str] = None, sms_gateway: Optional[str] = None, **kwargs):
        """This class is reponsible of sending a notification via sms.

        :param receivers: list of User
        :param template: the name of the template to user. Default None
        :param context: the context to be passed to template. Default None
        :param final_message: the final message to be sent as the notification content, must be sent if template is None, template is ignored if it is sent. Default None
        :param sms_gateway: the sms gateway to use. Default to None
        :param kwargs:
        """
        self.receivers: list = receivers
        self.template: Optional[str] = template
        self.context: dict = context
        self.threaded: bool = kwargs.get("threaded", False)
        self.final_message: Optional[str] = final_message
        # get the default sms gateway
        self.sms_gateway = get_settings('SMS::DEFAULT_GATEWAY') if sms_gateway is None else sms_gateway
        # get the sms gateway definition
        NOTIFIER_SMS_GATEWAY = get_settings('SMS')["GATEWAYS"][self.sms_gateway]
        # get the sms client to be used
        NOTIFIER_SMS_CLIENT = NOTIFIER_SMS_GATEWAY['CLIENT']
        # load the sms client
        module_name, class_name = NOTIFIER_SMS_CLIENT.rsplit(".", 1)
        module = importlib.import_module(module_name)
        assert hasattr(module, class_name), "class {} is not in {}".format(class_name, module_name)
        self.client_class = getattr(module, class_name)
        self.sms_class_options = NOTIFIER_SMS_GATEWAY

    def send(self):
        if self.threaded:
            t = Thread(target=self._send)
            t.setDaemon(True)
            t.start()
        else:
            self._send()

    def _send(self):
        get_user_number = import_attribute(get_settings("GET_USER_NUMBER"))

        try:
            for rec in self.receivers:
                ctx = self.context.copy()
                ctx["user"] = rec
                number = get_user_number(rec)
                if not number:
                    logger.warning(f"Can't find a number for {rec}, ignoring.")

                if self.final_message:
                    sms_content = self.final_message
                else:
                    sms_content = render_to_string("notifier/{}/sms.txt".format(self.template), ctx)

                self.client_class.send(number, sms_content, **self.sms_class_options)
        except:
            logger.error(traceback.format_exc())



import logging

from django.conf import settings
from twilio.rest import Client

from .base import BaseSmsClient

logger = logging.getLogger("notifier")


class TwilioClient(BaseSmsClient):

    @classmethod
    def send(cls, number: str, text: str, **kwargs):
        account = kwargs["ACCOUNT"]
        token = kwargs["TOKEN"]
        from_number = kwargs["FROM_NUMBER"]
        client = Client(account, token)
        res = client.messages.create(from_=from_number, to=number, body=text)
        return res



import json
from operator import mod

from django import VERSION as DJANGO_VERSION
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.deletion import CASCADE
from django.db.models.fields.related import ForeignKey, OneToOneField
from django.utils.translation import gettext as _

if DJANGO_VERSION[0] >= 3 and DJANGO_VERSION[1] >= 1:
    from django.db.models import JSONField
else:
    try:
        from django.contrib.postgres.fields import JSONField
    except:
        class JSONField(models.TextField):
            """Simple JSON field that stores python structures as JSON strings
            on database.
            """

            def from_db_value(self, value, *args, **kwargs):
                return self.to_python(value)

            def to_python(self, value):
                """
                Convert the input JSON value into python structures, raises
                django.core.exceptions.ValidationError if the data can't be converted.
                """
                if self.blank and not value:
                    return None
                if isinstance(value, str):
                    try:
                        return json.loads(value)
                    except Exception as e:
                        raise ValidationError(str(e))
                else:
                    return value

            def validate(self, value, model_instance):
                """Check value is a valid JSON string, raise ValidationError on
                error."""
                if isinstance(value, str):
                    super(JSONField, self).validate(value, model_instance)
                    try:
                        json.loads(value)
                    except Exception as e:
                        raise ValidationError(str(e))

            def get_prep_value(self, value):
                """Convert value to JSON string before save"""
                try:
                    return json.dumps(value)
                except Exception as e:
                    raise ValidationError(str(e))

            def value_from_object(self, obj):
                """Return value dumped to string."""
                val = super(JSONField, self).value_from_object(obj)
                return self.get_prep_value(val)

from .settings import NOTIFIER_AVAILABLE_MODES, NOTIFIER_DEFAULT_MODE

User = get_user_model()


class Notification(models.Model):

    id = models.BigAutoField(primary_key=True)
    user: models.ForeignKey = models.ForeignKey(
        User, models.CASCADE, null=True, blank=True, related_name="magic_notifications"
    )
    subject: models.CharField = models.CharField(max_length=255)
    text: models.TextField = models.TextField()
    type: models.CharField = models.CharField(max_length=30)
    sub_type: models.CharField = models.CharField(max_length=30, null=True, blank=True)
    link: models.CharField = models.CharField(_("The link associated"), max_length=255)
    image: models.ImageField = models.ImageField(upload_to="notifications", null=True, blank=True)
    actions: JSONField = JSONField(default=dict)
    data: JSONField = JSONField(default=dict)
    read: models.DateTimeField = models.DateTimeField(null=True, blank=True)
    sent: models.DateTimeField = models.DateTimeField(auto_now_add=True)
    mode: models.CharField = models.CharField(
        max_length=10, default=NOTIFIER_DEFAULT_MODE, choices=NOTIFIER_AVAILABLE_MODES
    )

    def __str__(self):
        if self.user and self.user.username:
            user_name = self.user.username
        else:
            user_name = ""
        return "{} Notif #{}".format(user_name, self.id)

    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)

    def mark_read(self):
        from django.utils import timezone
        self.read = timezone.now()
        self.save()


class NotifyProfile(models.Model):

    id = models.BigAutoField(primary_key=True)
    phone_number: models.CharField = models.CharField(max_length=20, null=True, blank=True)
    current_channel: models.CharField = models.CharField(max_length=255, null=True, blank=True)
    user: models.OneToOneField = models.OneToOneField(User, models.CASCADE)



import logging
import traceback
from typing import Optional, Union

from django.contrib.auth import get_user_model
from django.db import models

from magic_notifier.emailer import Emailer
from magic_notifier.pusher import Pusher
from magic_notifier.settings import NOTIFIER_THREADED
from magic_notifier.smser import ExternalSMS

User = get_user_model()

logger = logging.getLogger("notifier")


def notify(
    vias: list,
    subject: str = None,
    receivers: Union[str, list, models.QuerySet, models.Manager] = None,
    template: str = None,
    context: dict = None,
    final_message: str = None,
    email_gateway: str = 'default',
    sms_gateway: Optional[str] = None,
    files: list = None,
    threaded: bool = None,
):
    """This function send a notification via the method specified in parameter vias

    :param vias: accepted values are email,sms,push
    :param subject: the subject of the notification, ignored when send by sms
    :param receivers: it can be a list, queryset or manager of users. if a string is passed it must be *admins* to send to (super) admins, *staff* to send to staff only, *all* to all users, *all-staff* to all users minus staff and *all-admins* to all users excepted admins
    :param template: the name of the template to user. Default None
    :param context: the context to be passed to template. Note that the context is auto-filled with the current the notification is going under the key 'user'. Default None
    :param final_message: the final message to be sent as the notification content, must be sent if template is None, template is ignored if it is sent. Default None
    :param email_gateway: the email gateway to use. Default 'default'
    :param sms_gateway: the sms gateway to use. Default to None
    :param files: list of files to be sent. accept file-like objects, tuple, file path. Default None
    :param threaded: if True, the notification is sent in background else sent with the current thread. Default to NOTIFIER["THREADED"] settings
    :return:
    """
    logger.debug(f"Sending {subject} to {receivers} via {vias}")
    threaded = threaded if threaded is not None else NOTIFIER_THREADED
    context = {} if context is None else context

    assert subject, "subject not defined"


    if isinstance(receivers, str):
        if receivers in ["admins", "staff", "all", "all-staff", "all-admins"]:
            if receivers == "admins":
                receivers = User.objects.filter(is_superuser=True)
            elif receivers == "staff":
                receivers = User.objects.filter(is_staff=True)
            elif receivers == "all":
                receivers = User.objects.all()
            elif receivers == "all-staff":
                receivers = User.objects.exclude(is_staff=True)
            elif receivers == "all-admins":
                receivers = User.objects.exclude(is_superuser=True)
        else:
            raise ValueError(f"'{receivers}' is not an allowed value for receivers arguments")

    assert isinstance(receivers, (list, models.Manager, models.QuerySet)), f"receivers must be a list at this point not {receivers}"

    for via in vias:
        try:
            if via == "email":
                em = Emailer(
                    subject,
                    list(receivers),
                    template,
                    context,
                    email_gateway,
                    threaded=threaded,
                    final_message=final_message,
                    files=files
                )
                em.send()

            elif via == "sms":
                ex_sms = ExternalSMS(receivers,context, threaded=threaded,
                    template=template, final_message=final_message,
                    sms_gateway=sms_gateway)
                ex_sms.send()

            elif via == "push":
                assert template, "template variable can't be None or empty"

                pusher = Pusher(
                    subject, receivers, template, context, threaded=threaded
                )
                pusher.send()

            else:
                logger.error(f"Unknown sending method {via}")

        except:
            logger.error(traceback.format_exc())



import logging

import requests
from django.conf import settings

from .base import BaseSmsClient

logger = logging.getLogger("notifier")


class NexaSmsClient(BaseSmsClient):

    @classmethod
    def send(cls, number: str, text: str, **kwargs):
        sub_account = settings.NOTIFIER["SMS"]["GATEWAYS"]["NEXA"]["EMAIL"]
        sub_account_pass = settings.NOTIFIER["SMS"]["GATEWAYS"]["NEXA"]["PASSWORD"]
        senderid = settings.NOTIFIER["SMS"]["GATEWAYS"]["NEXA"]["SENDERID"]
        params = {
            "user": sub_account,
            "password": sub_account_pass,
            "senderid": senderid,
            "sms": text,
            "mobiles": number.replace('+', ''),
        }
        logger.info(f"Sending sms with data {params}")
        res = requests.post("https://smsvas.com/bulk/public/index.php/api/v1/sendsms", json=params)
        if(res.status_code != 200):
            logger.error(f"Failed to send the sms: {res.content}")
        else:
            logger.info(res.content)
        return res



from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand

from magic_notifier.notifier import notify

User = get_user_model()


class Command(BaseCommand):
    """The command `test_email_template` is used to test a template email.
    This is very useful in development."""

    def add_arguments(self, parser):
        parser.add_argument('template', type=str, help="The template to test")
        parser.add_argument('email', type=str, help="The email to use")
        parser.add_argument('-s', '--subject', type=str, default='Testing email template', required=False,
                            help="The subject of email. default to `Testing email template`")
        parser.add_argument('-a', '--account', type=str, default='default', required=False,
                            help="The smtp account. default to `default`")

    def handle(self, *args, **options):

        try:
            template = options['template']
            subject = options['subject']
            account = options['account']
            user_email = options['email']

            print(f"Sending email from template {template} to {user_email}")


            user = User(email=user_email, username=user_email)
            notify(["email"], subject=subject, email_gateway=account, receivers=[user], template=template,
                   context={})

            print('Done!')
        except Exception:
            import traceback
            traceback.print_exc()



# Generated by Django 3.2.15 on 2022-09-14 17:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('magic_notifier', '0002_auto_20220907_2112'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='subject',
            field=models.CharField(default='', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='notification',
            name='image',
            field=models.ImageField(blank=True, null=True, upload_to='notifications'),
        ),
    ]
